﻿using Android.App;
using Android.Content;
using Android.Views;
using Android.Widget;
using Com.Google.Android.Exoplayer2;
using Com.Google.Android.Exoplayer2.Source;
using Com.Google.Android.Exoplayer2.Trackselection;
using Com.Google.Android.Exoplayer2.UI;
using Com.Google.Android.Exoplayer2.Upstream;
using Com.Google.Android.Exoplayer2.Upstream.Cache;
using Java.Net;
using System;
using WoWonder.Activities.NativePost.Pages;
using WoWonder.Helpers.Utils;
using Uri = Android.Net.Uri;

namespace WoWonder.MediaPlayers.Exo
{
    public class ExoController
    {
        public VolumeState VolumeStateProvider = VolumeState.On;
        public enum VolumeState
        {
            On = 0,
            Off = 1
        }

        private readonly Activity ActivityContext;
        private readonly string Page;

        private IExoPlayer VideoPlayer, FullScreenVideoPlayer;
        private StyledPlayerView PlayerView, FullScreenPlayerView;
        private StyledPlayerControlView ControlView;

        private PreCachingExoPlayerVideo PreCachingExoPlayerVideo;

        private IDataSource.IFactory DataSourceFactory;
        private IDataSource.IFactory HttpDataSourceFactory;
        private PlayerEvents PlayerListener;

        private ImageView MVolumeIcon, MFullScreenIcon;
        private FrameLayout MFullScreenButton;

        private string FullScreenTag;
        private bool IsFullScreen;

        public ExoController(Activity context, string page = "")
        {
            try
            {
                ActivityContext = context;
                Page = page;
            }
            catch (Exception e)
            {
                Methods.DisplayReportResultTrack(e);
            }
        }

        public void SetPlayer(StyledPlayerView playerView, bool useController = true)
        {
            try
            {
                PlayerView = playerView;

                PreCachingExoPlayerVideo = new PreCachingExoPlayerVideo(ActivityContext);
                DefaultTrackSelector trackSelector = new DefaultTrackSelector(ActivityContext);
                ControlView = PlayerView.FindViewById<StyledPlayerControlView>(Resource.Id.exo_controller);

                VideoPlayer = new IExoPlayer.Builder(ActivityContext)?.SetTrackSelector(trackSelector)?.Build();
                PlayerListener = new PlayerEvents(this, ControlView);
                VideoPlayer?.AddListener(PlayerListener);

                PlayerView.UseController = useController;
                PlayerView.Player = VideoPlayer;
            }
            catch (Exception e)
            {
                Methods.DisplayReportResultTrack(e);
            }
        }

        public void SetPlayerControl(bool showFullScreen = true, bool isFullScreen = false)
        {
            try
            {
                if (ControlView != null)
                {
                    //Check All Views 
                    MVolumeIcon = ControlView.FindViewById<ImageView>(Resource.Id.exo_volume_icon);
                    MFullScreenIcon = ControlView.FindViewById<ImageView>(Resource.Id.exo_fullscreen_icon);
                    MFullScreenButton = ControlView.FindViewById<FrameLayout>(Resource.Id.exo_fullscreen_button);

                    switch (VolumeStateProvider)
                    {
                        case VolumeState.Off:
                            MVolumeIcon.SetImageResource(Resource.Drawable.ic_volume_off_grey_24dp);
                            VideoPlayer.Volume = 0f;
                            VolumeStateProvider = VolumeState.Off;
                            break;
                        case VolumeState.On:
                            MVolumeIcon.SetImageResource(Resource.Drawable.ic_volume_up_grey_24dp);
                            VideoPlayer.Volume = 1f;
                            VolumeStateProvider = VolumeState.On;
                            break;
                        default:
                            MVolumeIcon.SetImageResource(Resource.Drawable.ic_volume_off_grey_24dp);
                            VideoPlayer.Volume = 0f;
                            VolumeStateProvider = VolumeState.Off;
                            break;
                    }

                    MVolumeIcon.Click += ToggleVolumeOnClick;

                    if (!showFullScreen)
                    {
                        MVolumeIcon.Visibility = ViewStates.Gone;
                        MFullScreenButton.Visibility = ViewStates.Gone;
                    }
                    else
                    {
                        MFullScreenButton.Visibility = ViewStates.Visible;

                        MFullScreenButton.Click += MFullScreenButtonOnClick;

                        if (isFullScreen)
                        {
                            FullScreenTag = "true";
                            MFullScreenIcon.SetImageResource(Resource.Drawable.ic_action_ic_fullscreen_skrink);
                        }
                        else
                        {
                            FullScreenTag = "false";
                            MFullScreenIcon.SetImageResource(Resource.Drawable.ic_action_ic_fullscreen_expand);
                        }
                    }

                }
            }
            catch (Exception e)
            {
                Methods.DisplayReportResultTrack(e);
            }
        }

        public IMediaSource GetMediaSourceFromUrl(Uri uri, string tag)
        {
            try
            {
                if (DataSourceFactory == null)
                {
                    DefaultDataSource.Factory upstreamFactory = new DefaultDataSource.Factory(ActivityContext, GetHttpDataSourceFactory());
                    DataSourceFactory = BuildReadOnlyCacheDataSource(upstreamFactory, PreCachingExoPlayerVideo.GetCache());
                }

                IMediaSource src = new ProgressiveMediaSource.Factory(DataSourceFactory).CreateMediaSource(MediaItem.FromUri(uri));
                if (src?.MediaItem != null)
                    src.MediaItem.MediaId = tag;

                return src;
            }
            catch (Exception e)
            {
                Methods.DisplayReportResultTrack(e);
                return null!;
            }
        }

        private IDataSource.IFactory GetHttpDataSourceFactory()
        {
            if (HttpDataSourceFactory == null)
            {
                CookieManager cookieManager = new CookieManager();
                cookieManager.SetCookiePolicy(ICookiePolicy.AcceptOriginalServer);
                CookieHandler.Default = cookieManager;
                HttpDataSourceFactory = new DefaultHttpDataSource.Factory();
            }

            return HttpDataSourceFactory;
        }

        private CacheDataSource.Factory BuildReadOnlyCacheDataSource(IDataSource.IFactory upstreamFactory, ICache cache)
        {
            return new CacheDataSource.Factory()?.SetCache(cache)?.SetUpstreamDataSourceFactory(upstreamFactory)?.SetCacheWriteDataSinkFactory(null)?.SetFlags(CacheDataSource.FlagIgnoreCacheOnError);
        }

        public void FirstPlayVideo(Uri uri)
        {
            try
            {
                var videoSource = GetMediaSourceFromUrl(uri, "normal");

                if (PlayerSettings.EnableOfflineMode && uri.ToString()!.Contains("http"))
                {
                    PreCachingExoPlayerVideo.CacheVideosFiles(uri);
                    videoSource = new ProgressiveMediaSource.Factory(PreCachingExoPlayerVideo.CacheDataSourceFactory).CreateMediaSource(MediaItem.FromUri(uri));
                }

                VideoPlayer.SetMediaSource(videoSource, true);
                VideoPlayer.Prepare();
                VideoPlayer.PlayWhenReady = true;
                VideoPlayer.SeekTo(0, 0);
            }
            catch (Exception exception)
            {
                Methods.DisplayReportResultTrack(exception);
            }
        }

        public void FirstPlayVideo(Uri uri, int videoDuration)
        {
            try
            {
                var videoSource = GetMediaSourceFromUrl(uri, "normal");

                if (PlayerSettings.EnableOfflineMode && uri.ToString()!.Contains("http"))
                {
                    PreCachingExoPlayerVideo.CacheVideosFiles(uri);
                    videoSource = new ProgressiveMediaSource.Factory(PreCachingExoPlayerVideo.CacheDataSourceFactory).CreateMediaSource(MediaItem.FromUri(uri));
                }

                VideoPlayer.SetMediaSource(videoSource);
                VideoPlayer.Prepare();
                VideoPlayer.PlayWhenReady = true;
                VideoPlayer.SeekTo(videoDuration);
            }
            catch (Exception exception)
            {
                Methods.DisplayReportResultTrack(exception);
            }
        }

        public void PlayVideo()
        {
            try
            {
                if (PlayerView?.Player != null && PlayerView.Player.PlaybackState == IPlayer.StateReady && !PlayerView.Player.PlayWhenReady)
                    PlayerView.Player.PlayWhenReady = true;
            }
            catch (Exception exception)
            {
                Methods.DisplayReportResultTrack(exception);
            }
        }

        public void StopVideo()
        {
            try
            {
                if (PlayerView?.Player != null && PlayerView.Player.PlayWhenReady)
                    PlayerView.Player.PlayWhenReady = false;
            }
            catch (Exception exception)
            {
                Methods.DisplayReportResultTrack(exception);
            }
        }

        public void ReleaseVideo()
        {
            try
            {
                StopVideo();
                PlayerView?.Player?.Stop();

                if (VideoPlayer != null)
                {
                    VideoPlayer.Release();
                    VideoPlayer = null!;
                }
            }
            catch (Exception exception)
            {
                Methods.DisplayReportResultTrack(exception);
            }
        }

        public StyledPlayerView GetPlayerView()
        {
            return PlayerView;
        }

        public IExoPlayer GetExoPlayer()
        {
            return VideoPlayer;
        }

        public void ToggleExoPlayerKeepScreenOnFeature(bool keepScreenOn)
        {
            try
            {
                if (PlayerView != null)
                {
                    PlayerView.KeepScreenOn = keepScreenOn;
                }

                if (FullScreenPlayerView != null)
                {
                    FullScreenPlayerView.KeepScreenOn = keepScreenOn;
                }
            }
            catch (Exception e)
            {
                Methods.DisplayReportResultTrack(e);
            }
        }

        #region FullScreen

        private void MFullScreenButtonOnClick(object sender, EventArgs e)
        {
            try
            {
                if (FullScreenTag == "false")
                {
                    InitFullscreenDialog("Open");
                }
                else if (FullScreenTag == "true")
                {
                    InitFullscreenDialog("Close");
                }
            }
            catch (Exception exception)
            {
                Methods.DisplayReportResultTrack(exception);
            }
        }

        public void InitFullscreenDialog(string action)
        {
            try
            {
                if (action == "Open")
                {
                    Intent intent = new Intent(ActivityContext, typeof(VideoFullScreenActivity));
                    intent.PutExtra("type", "auto");
                    intent.PutExtra("page", Page);
                    ActivityContext.StartActivityForResult(intent, 2000);
                    IsFullScreen = true;
                }
                else
                {
                    Intent intent = new Intent();
                    VideoFullScreenActivity.Instance?.SetResult(Result.Ok, intent);
                    VideoFullScreenActivity.Instance?.Finish();
                    IsFullScreen = false;
                }
            }
            catch (Exception e)
            {
                Methods.DisplayReportResultTrack(e);
            }
        }

        public void SetFullScreenPlayerView(StyledPlayerView playerView)
        {
            try
            {
                FullScreenPlayerView = playerView;

                ControlView = FullScreenPlayerView.FindViewById<StyledPlayerControlView>(Resource.Id.exo_controller);

                FullScreenVideoPlayer = new IExoPlayer.Builder(ActivityContext)?.SetTrackSelector(new DefaultTrackSelector(ActivityContext))?.Build();
                var playerListener = new PlayerEvents(this, ControlView);
                FullScreenVideoPlayer?.AddListener(playerListener);

                FullScreenPlayerView.UseController = true;
                FullScreenPlayerView.Player = VideoPlayer;
                FullScreenPlayerView.Player.PlayWhenReady = true;
            }
            catch (Exception e)
            {
                Methods.DisplayReportResultTrack(e);
            }
        }

        public void PlayFullScreen()
        {
            try
            {
                if (FullScreenPlayerView != null)
                {
                    FullScreenPlayerView.Player = VideoPlayer;
                    if (FullScreenPlayerView.Player != null) FullScreenPlayerView.Player.PlayWhenReady = true;

                    FullScreenTag = "true";
                    MFullScreenIcon.SetImageDrawable(ActivityContext.GetDrawable(Resource.Drawable.ic_action_ic_fullscreen_skrink));
                }
            }
            catch (Exception exception)
            {
                Methods.DisplayReportResultTrack(exception);
            }
        }

        public void RestartPlayAfterShrinkScreen()
        {
            try
            {
                PlayerView.Player = null!;
                if (FullScreenPlayerView != null)
                {
                    PlayerView.Player = FullScreenPlayerView.Player;
                    PlayerView.Player.PlayWhenReady = true;
                    PlayerView.RequestFocus();
                    PlayerView.Visibility = ViewStates.Visible;

                    FullScreenTag = "false";
                    MFullScreenIcon.SetImageDrawable(ActivityContext.GetDrawable(Resource.Drawable.ic_action_ic_fullscreen_expand));

                    FullScreenPlayerView.Player = null;
                }
            }
            catch (Exception exception)
            {
                Methods.DisplayReportResultTrack(exception);
            }
        }

        #endregion

        #region Volume

        private void ToggleVolumeOnClick(object sender, EventArgs e)
        {
            try
            {
                switch (VolumeStateProvider)
                {
                    case VolumeState.Off:
                        Console.WriteLine("togglePlaybackState: enabling volume.");
                        SetVolumeControl(VolumeState.On);
                        break;
                    case VolumeState.On:
                        Console.WriteLine("togglePlaybackState: disabling volume.");
                        SetVolumeControl(VolumeState.Off);
                        break;
                }
            }
            catch (Exception exception)
            {
                Methods.DisplayReportResultTrack(exception);
            }
        }

        public void SetVolumeControl(VolumeState state)
        {
            try
            {
                VolumeStateProvider = state;
                switch (state)
                {
                    case VolumeState.Off:
                        MVolumeIcon.SetImageResource(Resource.Drawable.ic_volume_off_grey_24dp);
                        VideoPlayer.Volume = 0f;
                        break;
                    case VolumeState.On:
                        MVolumeIcon.SetImageResource(Resource.Drawable.ic_volume_up_grey_24dp);
                        VideoPlayer.Volume = 1f;
                        break;
                }
            }
            catch (Exception e)
            {
                Methods.DisplayReportResultTrack(e);
            }
        }

        #endregion
    }
}